package org.snmp4j.transport;

import org.snmp4j.smi.Address;

import java.net.ServerSocket;

public abstract class AbstractConnectionOrientedTransportMapping<A extends Address>
        extends AbstractTransportMapping<A> implements ConnectionOrientedTransportMapping<A> {

    protected boolean serverEnabled = false;

    protected synchronized void timeoutSocket(AbstractSocketEntry<A> entry) {
        if ((connectionTimeout > 0) && (getSocketCleaner() != null)) {
            SocketTimeout<A> socketTimeout = new SocketTimeout<A>(this, entry);
            entry.setSocketTimeout(socketTimeout);
            getSocketCleaner().schedule(socketTimeout, connectionTimeout);
        }
    }


    /**
     * Gets the connection timeout. This timeout specifies the time a connection
     * may be idle before it is closed.
     *
     * @return long
     * the idle timeout in milliseconds.
     */
    public long getConnectionTimeout() {
        return connectionTimeout;
    }

    /**
     * Sets the connection timeout. This timeout specifies the time a connection
     * may be idle before it is closed.
     *
     * @param connectionTimeout
     *         the idle timeout in milliseconds. A zero or negative value will disable
     *         any timeout and connections opened by this transport mapping will stay
     *         opened until they are explicitly closed.
     */
    public void setConnectionTimeout(long connectionTimeout) {
        this.connectionTimeout = connectionTimeout;
    }

    /**
     * Sets optional server socket options. The default implementation does
     * nothing.
     *
     * @param serverSocket
     *         the {@link ServerSocket} to apply additional non-default options.
     */
    protected void setSocketOptions(ServerSocket serverSocket) {
    }

    @Override
    public boolean isServerEnabled() {
        return serverEnabled;
    }

    @Override
    public void setServerEnabled(boolean serverEnabled) {
        this.serverEnabled = serverEnabled;
    }
}
